==> ngcsdk-3.41.3-py3-none-any.whl: basecommand/api/aces.py <==
#
# Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation and its affiliates
# (the "Company") and all right, title, and interest in and to the software
# product, including all associated intellectual property rights, are and
# shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.

from itertools import chain
from typing import Literal, Optional

from ngcbpc.api.configuration import Configuration
from ngcbpc.api.pagination import pagination_helper
from ngcbpc.constants import API_VERSION, PAGE_SIZE
from ngcbpc.errors import NgcException
from ngcbpc.util.utils import extra_args, url_encode
from ngccli.data.api.AceListResponse import AceListResponse
from ngccli.data.api.AceResponse import AceResponse

_NO_ACE_RESOURCES_FOUND_ERROR_MESSAGE = """\
This ACE has not been configured, and as a result, no usage information is available.

Please report this issue: https://nvcrm.my.site.com/ESPCommunity/s/create-case"""


class AceAPI:
    def __init__(self, connection):
        self.connection = connection
        self.config = Configuration()

    @staticmethod
    def _construct_url(org_name, ace_name=None, team_name=None):
        """Constructs ace url depending on given parameters."""

        base_method = "{api_version}/org/{org_name}".format(api_version=API_VERSION, org_name=org_name)
        if team_name:
            base_method = f"{base_method}/team/{team_name}"
        if ace_name:
            base_method = "{url_method}/aces/{ace_name}".format(url_method=base_method, ace_name=ace_name)
        else:
            base_method = "{url_method}/aces".format(url_method=base_method)
        return base_method

    def get_aces(self, org_name, team_name=None):
        """Get list of ACEs. Given team name filters on basis of it."""

        query = "{url}?page_size={page_size}".format(
            url=self._construct_url(org_name, team_name=team_name), page_size=PAGE_SIZE
        )
        return chain(
            *[
                AceListResponse(res).aces
                for res in pagination_helper(
                    self.connection, query, org_name=org_name, team_name=team_name, operation_name="get aces"
                )
                if AceListResponse(res).aces
            ]
        )

    # TODO: Remove ace_id once backwards compatibility support is removed
    def get_ace_details(self, org_name, ace_name=None, ace_id=None, team_name=None):
        """Gets detail of an ACE."""

        if not ace_name and not ace_id:
            raise NgcException("Please provide ACE name.")

        if ace_name:
            ace_name = url_encode(ace_name)
        else:
            ace_name = ace_id

        response = self.connection.make_api_request(
            "GET",
            self._construct_url(org_name, ace_name, team_name),
            auth_org=org_name,
            auth_team=team_name,
            operation_name="get ace details",
        )
        return AceResponse(response).ace

    @extra_args
    def list(self, org: Optional[str] = None, team: Optional[str] = None):
        """List ACEs in the org."""
        self.config.validate_configuration()
        org = org or self.config.org_name
        team = team or self.config.team_name
        ace_list = self.get_aces(org_name=org, team_name=team)
        return ace_list

    @extra_args
    def info(self, ace: str, org: Optional[str] = None, team: Optional[str] = None):
        """Gets detail of an ACE."""
        self.config.validate_configuration()
        org = org or self.config.org_name
        team = team or self.config.team_name
        ace_details = self.get_ace_details(org_name=org, ace_name=ace, team_name=team)
        return ace_details

    @extra_args
    def usage(
        self,
        ace: str,
        *,
        org: Optional[str] = None,
        team: Optional[str] = None,
        resource_type: Optional[Literal["GPU", "CPU", "MIG"]] = None,
        only_unavailable: bool = False,
    ):
        """Get resource usage information about an ACE."""
        self.config.validate_configuration()
        org = org or self.config.org_name
        team = team or self.config.team_name

        endpoint = self._construct_url(org, ace, team) + "/usage"

        response = self.connection.make_api_request(
            "GET",
            endpoint,
            auth_org=org,
            auth_team=team,
            operation_name="get ace usage",
        )
        items = response["resourceUsages"]
        if not items:
            # No items means the ACE isn't configured. That's bad and will likely need our intervention
            raise NgcException(_NO_ACE_RESOURCES_FOUND_ERROR_MESSAGE)

        if only_unavailable:
            items = [item for item in items if item.get("unavailableCount")]
        if resource_type:
            items = [item for item in items if item["type"] == resource_type]

        return items

==> ngcsdk-3.43.0-py3-none-any.whl: basecommand/api/aces.py <==
#
# Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation and its affiliates
# (the "Company") and all right, title, and interest in and to the software
# product, including all associated intellectual property rights, are and
# shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.

from itertools import chain
from typing import Literal, Optional

from ngcbpc.api.configuration import Configuration
from ngcbpc.api.pagination import pagination_helper
from ngcbpc.constants import API_VERSION, PAGE_SIZE
from ngcbpc.data.api.AceListResponse import AceListResponse
from ngcbpc.data.api.AceResponse import AceResponse
from ngcbpc.errors import NgcException
from ngcbpc.util.utils import extra_args, url_encode

_NO_ACE_RESOURCES_FOUND_ERROR_MESSAGE = """\
This ACE has not been configured, and as a result, no usage information is available.

Please report this issue: https://nvcrm.my.site.com/ESPCommunity/s/create-case"""


class AceAPI:
    def __init__(self, connection):
        self.connection = connection
        self.config = Configuration()

    @staticmethod
    def _construct_url(org_name, ace_name=None, team_name=None):
        """Constructs ace url depending on given parameters."""

        base_method = "{api_version}/org/{org_name}".format(api_version=API_VERSION, org_name=org_name)
        if team_name:
            base_method = f"{base_method}/team/{team_name}"
        if ace_name:
            base_method = "{url_method}/aces/{ace_name}".format(url_method=base_method, ace_name=ace_name)
        else:
            base_method = "{url_method}/aces".format(url_method=base_method)
        return base_method

    def get_aces(self, org_name, team_name=None):
        """Get list of ACEs. Given team name filters on basis of it."""

        query = "{url}?page_size={page_size}".format(
            url=self._construct_url(org_name, team_name=team_name), page_size=PAGE_SIZE
        )
        return chain(
            *[
                AceListResponse(res).aces
                for res in pagination_helper(
                    self.connection, query, org_name=org_name, team_name=team_name, operation_name="get aces"
                )
                if AceListResponse(res).aces
            ]
        )

    # TODO: Remove ace_id once backwards compatibility support is removed
    def get_ace_details(self, org_name, ace_name=None, ace_id=None, team_name=None):
        """Gets detail of an ACE."""

        if not ace_name and not ace_id:
            raise NgcException("Please provide ACE name.")

        if ace_name:
            ace_name = url_encode(ace_name)
        else:
            ace_name = ace_id

        response = self.connection.make_api_request(
            "GET",
            self._construct_url(org_name, ace_name, team_name),
            auth_org=org_name,
            auth_team=team_name,
            operation_name="get ace details",
        )
        return AceResponse(response).ace

    @extra_args
    def list(self, org: Optional[str] = None, team: Optional[str] = None):
        """List ACEs in the org."""
        self.config.validate_configuration()
        org = org or self.config.org_name
        team = team or self.config.team_name
        ace_list = self.get_aces(org_name=org, team_name=team)
        return ace_list

    @extra_args
    def info(self, ace: str, org: Optional[str] = None, team: Optional[str] = None):
        """Gets detail of an ACE."""
        self.config.validate_configuration()
        org = org or self.config.org_name
        team = team or self.config.team_name
        ace_details = self.get_ace_details(org_name=org, ace_name=ace, team_name=team)
        return ace_details

    @extra_args
    def usage(
        self,
        ace: str,
        *,
        org: Optional[str] = None,
        team: Optional[str] = None,
        resource_type: Optional[Literal["GPU", "CPU", "MIG"]] = None,
        only_unavailable: bool = False,
    ):
        """Get resource usage information about an ACE."""
        self.config.validate_configuration()
        org = org or self.config.org_name
        team = team or self.config.team_name

        endpoint = self._construct_url(org, ace, team) + "/usage"

        response = self.connection.make_api_request(
            "GET",
            endpoint,
            auth_org=org,
            auth_team=team,
            operation_name="get ace usage",
        )
        items = response["resourceUsages"]
        if not items:
            # No items means the ACE isn't configured. That's bad and will likely need our intervention
            raise NgcException(_NO_ACE_RESOURCES_FOUND_ERROR_MESSAGE)

        if only_unavailable:
            items = [item for item in items if item.get("unavailableCount")]
        if resource_type:
            items = [item for item in items if item["type"] == resource_type]

        return items
